﻿using Bulldozer.Entities;
using Bulldozer.MovementStrategy;

namespace Bulldozer.DrawingObjects
{
    /// <summary>
    ///  Класс отрисовки и перемещения объекта-сущности
    /// </summary>
    public class DrawingBulldozer
    {
        /// <summary>
        ///  Класс-сущность
        /// </summary>
        public EntityBulldozer? EntityBulldozer { get; protected set; }
        /// <summary>
        ///  Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        ///  Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        ///  Координата прорисовки по оси X
        /// </summary>
        protected int _startPosX;
        /// <summary>
        ///  Координата прорисовки по оси Y
        /// </summary>
        protected int _startPosY;
        /// <summary>
        ///  Ширина бульдозера
        /// </summary>
        protected readonly int _bulldozerWidth = 150;
        /// <summary>
        ///  Высота бульдозера
        /// </summary>
        protected readonly int _bulldozerHeight = 60;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawingBulldozer(int speed, double weight, Color bodyColor, Color additionalColor, int width, int height)
        {
                _pictureWidth = width;
                _pictureHeight = height;
                EntityBulldozer = new EntityBulldozer(speed, weight, bodyColor, additionalColor);
        }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <param name="carWidth">Ширина прорисовки бульдозера</param>
        /// <param name="carHeight">Высота прорисовки бульдозера</param>
        protected DrawingBulldozer(int speed, double weight, Color bodyColor, Color additionalColor, int width, int height, int bulldozerWidth, int bulldozerHeight)
        {
                _pictureWidth = width;
                _pictureHeight = height;
                _bulldozerWidth = bulldozerWidth;
                _bulldozerHeight = bulldozerHeight;
                EntityBulldozer = new EntityBulldozer(speed, weight, bodyColor, additionalColor);
        }

        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>

        public void SetPosition(int x, int y)
        {
            // Проверка границ
            if (x < 0) x = 0;
            if (y < 0) y = 0;
            if (x > _pictureWidth - _bulldozerWidth) x = _pictureWidth - _bulldozerWidth;
            if (y > _pictureHeight - _bulldozerHeight) y = _pictureHeight - _bulldozerHeight;

            _startPosX = x;
            _startPosY = y;
        }

        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _bulldozerWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _bulldozerHeight;
        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityBulldozer == null)
            {
                return false;
            }
            return direction switch
            {
                // влево
                DirectionType.Left => _startPosX - EntityBulldozer.Step > 0,
                // вверх
                DirectionType.Up => _startPosY - EntityBulldozer.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + EntityBulldozer.Step < _pictureWidth,
                // вниз
                DirectionType.Down => _startPosY + EntityBulldozer.Step < _pictureHeight,
                _ => false,
            };
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveBulldozer(DirectionType direction)
        {
            if (!CanMove(direction) || EntityBulldozer == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityBulldozer.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityBulldozer.Step;
                    break;
                // вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityBulldozer.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityBulldozer.Step;
                    break;
            }
        }
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawBulldozer(Graphics g)
        {
            if (EntityBulldozer == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush bodyBrush = new SolidBrush(EntityBulldozer.BodyColor);
            Brush additionalBrush = new SolidBrush(EntityBulldozer.AdditionalColor);
            /// <summary>
            /// Заливка гусеницы бульдозера 
            /// </summary>


            Brush brGray = new SolidBrush(Color.Gray);
            g.FillEllipse(brGray, _startPosX + 17, _startPosY + 24, 119, 40); //Гусеница
            g.FillEllipse(additionalBrush, _startPosX + 20, _startPosY + 35, 20, 20); // Левое колесо гусеницы
            g.FillEllipse(additionalBrush, _startPosX + 115, _startPosY + 35, 20, 20); // Правое колесо гусеницы
            g.FillEllipse(additionalBrush, _startPosX + 50, _startPosY + 45, 10, 10); // 1 центральное колесо гусеницы
            g.FillEllipse(additionalBrush, _startPosX + 70, _startPosY + 45, 10, 10); // 2 центральное колесо гусеницы
            g.FillEllipse(additionalBrush, _startPosX + 90, _startPosY + 45, 10, 10); // 3 центральное колесо гусеницы
            
            


            /// <summary>
            /// Отрисовка границ бульдозера
            /// </summary> 
            g.DrawEllipse(pen, _startPosX + 17, _startPosY + 24, 119, 40);
            g.DrawEllipse(pen, _startPosX + 20, _startPosY + 35, 20, 20); // Левое колесо гусеницы
            g.DrawEllipse(pen, _startPosX + 115, _startPosY + 35, 20, 20); // Правое колесо гусеницы
            g.DrawEllipse(pen, _startPosX + 50, _startPosY + 45, 10, 10); // 1 центральное колесо гусеницы
            g.DrawEllipse(pen, _startPosX + 70, _startPosY + 45, 10, 10); // 2 центральное колесо гусеницы
            g.DrawEllipse(pen, _startPosX + 90, _startPosY + 45, 10, 10); // 3 центральное колесо гусеницы


            /// <summary>
            /// Кузов бульдозера
            /// </summary>
            g.FillRectangle(bodyBrush, _startPosX + 102, _startPosY, 28, 24); //кабина
            g.FillRectangle(bodyBrush, _startPosX + 17, _startPosY + 24, 119, 18); // основная часть
            g.FillRectangle(bodyBrush, _startPosX + 30, _startPosY, 10, 24); // выхлопная труба

            /// Корпус
            /// </summary>
            g.DrawRectangle(pen, _startPosX + 102, _startPosY, 28, 24); //кабина
            g.DrawRectangle(pen, _startPosX + 17, _startPosY + 24, 119, 18); // основная часть
            g.DrawRectangle(pen, _startPosX + 30, _startPosY, 10, 24); // выхлопная труба
        }
        /// <summary>
        /// Получение объекта IMoveableObject из объекта DrawingBulldozer
        /// </summary>
        public IMoveableObject GetMoveableObject => new DrawingObjectBulldozer(this);
    }
    
}

